# Calibrated Data is sensors calibrated data. It gives calibrated data of
# each IMU. The input for processing comes from the main script. The
# Calibrated Data packets are of 106 bytes containing 4 bytes each for ax, ay,
# az, gx, gy, gz, for each IMU. Time stamp for 4 bytes, start code of 1
# byte,  packet number of 2 bytes payload length for 1 byte and checksum
# for 2 bytes.
#
#     Date     |    Review    |      Author      |  Comments
# -------------+--------------+------------------+----------------
#   10-07-2016 |    0.0       |   Rahul Tiwari   | Initial Release
#   21-08-2016 |    0.1       |   jaya sandhya M | included checksum check
#   01-08-2016 |    0.2       |   Ajit/Vijay     | Integrated BLE / USB connectivities
#
# (c) All Rights Reserved, GT Silicon Pvt Ltd, 2016

import Queue
import os.path
import thread
import binascii
from utilities import *
from collections import deque
from pyqtgraph.Qt import QtGui


d_graph = 'real time'                     # Display graph  #'Real Time'#'Off Line'
log_data = 0                              # Save lag data in file write 1 else 0
dlog_file = 'data/bin.txt'                # Data log path directory
out_rate = float(31.25)                   # Write Hz rate
conn_params = ('COM22', 115200)    # Write here serial port on your device
conn_type = 'usb'
# conn_params = ('00:A0:50:E4:74:C6', 115200)  # Write hare serial port on your device
# conn_type = 'ble'
runtime = float(100)                         # Enter time in second

MAX_FREQ = 250                            # Max data rate transmission
g_value = 9.81                            # gravity value set
NUM_AXIS = 9  # number of axis e.g ax, ay, az, gx, gy, gz , mx, my and mz
FUSE_ITEM = 1  # number of fused item e.g. timestamp
pi = [0, 0, 0, 15]  # select imu
# select acc, gyro, and magno [a0i a1i a2i g0i g1i g2i m0i m1i m2i] for each imu

select_acm = ['111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111',
              '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111',
              '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111',
              '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111', '111111111']


width = int(1600 - 500)
height = int(900 - 200)
app = None


def get_header(pkt_h):  # to get the packet header
    return struct.unpack("!BHB", pkt_h[0:4])


def get_time(pkt_t):  # to get the time stamp from the packet
    return struct.unpack("!L", pkt_t[4:8])[0]


def get_inertial_data(payload):  # to get the values from the paayload
    return struct.unpack("!ffffffffffffffffffffffffffffffffffff", payload)


# **** making empty lists to store values ****


pkt_number = []
time_stamps = []

d_graph = d_graph.upper()

if d_graph not in ['OFF LINE', 'REAL TIME']:
    tkMessageBox.showinfo("Error",
                          "dgraph's value is not correct."
                          "\nPlease enter the correct value e.g. 'Off Line' or 'Real Time'")
    print "dgraph's value is not correct. Please enter correct value e.g. 'Off Line' or 'Real Time'"
    sys.exit(-1)

if d_graph == 'OFF LINE' and log_data != 1:
    log_data = 1

binary_string = convert_bytes_to_string(pi)[::-1]  # reverse the returned string
num_of_selected_imu = binary_string.count('1')

connectivity_obj = get_connectivity(conn_type)

if connectivity_obj is None:
    tkMessageBox.showerror("Alert", "%s\nPlease give input connectivity type e.g. USB or WiFi or  BLE")
    sys.exit(1)

# Open serial port
try:
    com = connectivity_obj.open(conn_params)
except Exception as e:
    tkMessageBox.showerror("oops", "%s\nPlease restart the device and com port and try again" % e.message)
    sys.exit(1)

pkts = 0
count = 0
start = time.time()

queue = Queue.Queue()
temp_filename = None
temp_file = None
outfile = None
scale_acc = 1.0
scale_gyro = 57.325
scale_magn = 0.6

# generating the command
out_rate_ = MAX_FREQ / float(out_rate)
hex_val = [0x01, 0x02, 0x3, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f]
out_rate_cmd = hex_val[get_ratedevider(out_rate_)]
# cmd = [0x18, pi[0], pi[1], pi[2], pi[3], out_rate_cmd]
cmd = [0x31, 0x18, 0x1a, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
       0x63]  # command to starting read inertial and magnetometer
print "cmd1: ", cmd
connectivity_obj.send(cmd)
cmd = [0x21, 0x25, 0x26, 0x27, 0x28, 0xb9, 0xba, 0xbb, 0xbc, out_rate_cmd]
cs = cal_checksum_cmd(cmd)
cmd.append(cs[0])
cmd.append(cs[1])
print "cmd2: " + str(cmd)
connectivity_obj.send(cmd)
# *** receiving acknowledgement ***
ack = connectivity_obj.receive(8)
print "ack", ack

if os.path.isfile("stop"):
    os.remove("stop")

'''  RealTime Data - Start  '''
all_line = []
pXX_acm = []
all_imu_data = []
for i_imu in range(0, num_of_selected_imu * NUM_AXIS):
    all_imu_data.append([])

plot_option = get_plot_options(select_acm, binary_string)

if d_graph == 'OFF LINE' and log_data != 1:
    log_data = 1
if d_graph == 'REAL TIME':
    x_axis_value = int(int(get_file_value('data/xaxis.txt')) * float(out_rate))
    # *** queues for real time graph plotting ***
    app = QtGui.QApplication([])
    mw = QtGui.QMainWindow()
    win = pg.GraphicsWindow(title="Raw Data")
    mw.resize(width, height)
    cw = QtGui.QWidget()
    mw.setCentralWidget(cw)
    g = QtGui.QVBoxLayout()
    cw.setLayout(g)
    mw.setWindowTitle('pyqtgraph example: Plotting')

    list_pt_graph = []
    num_plot = 0
    for options in plot_option:
        # create subplot for accelerometer , gyroscope and magnetometer if selected
        if num_plot > 0:
            win.nextRow()
        num_plot += 1

        pt_desc = plot_descs[options]
        pt_graph = win.addPlot(title=pt_desc[0])
        pt_graph.setLabel('left', pt_desc[1], units=pt_desc[2])
        pt_graph.showGrid(x=True, y=True)
        list_pt_graph.append(pt_graph)

    list_pt_graph[-1].setLabel('bottom', 'TimeSpan : %d' % int(get_file_value('data/xaxis.txt')), units='seconds')

    for i_imu in range(0, num_of_selected_imu * NUM_AXIS):
        all_line.append([])
        pXX_acm.append(None)

    s_cntry = 0
    color = ['b', 'r', 'g']  # blue for x axis, red color for y axis and green color for z axis
    for s_imu in range(0, len(binary_string)):  # plot each axis from each imu if selected
        if binary_string[s_imu] == '1':
            acc_gyro_mag = select_acm[s_imu]
            for s_a_g in range(0, NUM_AXIS):
                plt_index = s_cntry * NUM_AXIS + s_a_g
                all_line[plt_index] = None
                if acc_gyro_mag[s_a_g] == '0':
                    continue
                all_line[plt_index] = deque([0] * x_axis_value, x_axis_value)
                which_colour = s_a_g % 3
                which_plt = int(s_a_g / 3)
                try:
                    pXX_acm[plt_index] = list_pt_graph[which_plt].plot(all_imu_data[plt_index], pen=color[which_colour])
                except IndexError as e:
                    # print e
                    pXX_acm[plt_index] = list_pt_graph[which_plt - 1].plot(all_imu_data[plt_index],
                                                                           pen=color[which_colour])
            s_cntry += 1

''' RealTime Data - End  '''

if log_data:
    try:
        temp_filename = 'temp_' + id_generator()
        temp_file = open(temp_filename, 'w')
        outfile = create_logfile(binary_string, select_acm, dlog_file)
        thread.start_new_thread(insert_data_logfile, (outfile, queue, num_of_selected_imu, NUM_AXIS, FUSE_ITEM,
                                                      temp_file, select_acm, binary_string))
    except Exception as e:
        print e
        print "Error: unable to start thread"
        sys.exit(-1)

pkt_size = 150
pkt = connectivity_obj.receive(pkt_size)

while time.time() - start < runtime:  # to run th process only for the runtime
    if not os.path.isfile("stop"):
        try:
            # print time.time() - start
            pkt = binascii.unhexlify(pkt)
            s1 = pkt.encode("hex")
            (start_code, pkt_num, payload_length) = get_header(pkt)
            if start_code == 0xAA and get_checksum(pkt) == cal_checksum(pkt):
                # time_stamp = get_time(pkt)
                vals = get_inertial_data(pkt[4:pkt_size - 2])
                pkt_number.append(pkt_num)
                data = [pkt_num]
                # Storing each IMUS values
                for i in range(0, num_of_selected_imu):
                    data.append(round(vals[6 * i + 0] * scale_acc, 3))
                    data.append(round(vals[6 * i + 1] * scale_acc, 3))
                    data.append(round(vals[6 * i + 2] * scale_acc, 3))
                    data.append(round(vals[6 * i + 3] * scale_gyro, 3))
                    data.append(round(vals[6 * i + 4] * scale_gyro, 3))
                    data.append(round(vals[6 * i + 5] * scale_gyro, 3))
                    k = 6 * num_of_selected_imu + 3 * i
                    data.append(round(vals[k + 0] * scale_magn, 3))
                    data.append(round(vals[k + 1] * scale_magn, 3))
                    data.append(round(vals[k + 2] * scale_magn, 3))

                if d_graph == 'REAL TIME':
                    s_cntry = 0
                    for s_imu in range(0, len(binary_string)):  # plot each axis from each imu if selected
                        if binary_string[s_imu] == '1':
                            acc_gyro_mag = select_acm[s_imu]
                            for s_a_g in range(0, NUM_AXIS):
                                if acc_gyro_mag[s_a_g] == '0':
                                    continue
                                which_axis = s_a_g % 3
                                which_plt = int(s_a_g / 3)
                                plt_index = s_cntry * NUM_AXIS + s_a_g
                                all_line[plt_index].appendleft(data[1 + plt_index])
                                all_line[plt_index].pop()
                                pXX_acm[plt_index].setData(all_line[plt_index])
                            s_cntry += 1
                    app.processEvents()

                if log_data:
                    queue.put(data)

                pkt = connectivity_obj.receive(pkt_size)
                pkts += 1
                count = 0
            elif re.search(b'[\d|\w]+aa.*', s1):  # search and find new packet
                lst = re.findall(b'[\d|\w]+(aa.*)', s1)
                str_rem = lst[0]
                length = len(str_rem) / 2
                pkt_rem = pkt[-length:]
                new_len = pkt_size - length
                pkt = connectivity_obj.receive(new_len)
                pkt = pkt_rem.encode("hex") + pkt
            else:  # exit the code if the header is detectig wrong
                pkt = connectivity_obj.receive(pkt_size)
                # exit the code if the packet is detectig wrong continuously for more than 5 times
                count += 1
                if count > 5:
                    count = 0
                    tkMessageBox.showinfo("Oops",
                                          "Something went wrong please restart the device and run the process again !")
                    sys.exit(1)
        except TypeError as e:
            s2 = ''
            for chk in range(0, len(pkt)):
                if is_hex(pkt[chk]) is True:
                    s2 += pkt[chk]
            if re.search(b'[\d|\w]+aa.*', s2):  # search and find new packet
                lst = re.findall(b'[\d|\w]+(aa.*)', s2)
                str_rem = lst[0]
                length = len(str_rem) / 1
                pkt_rem = pkt[-length:]
                new_len = pkt_size * 2 - length
                pkt = connectivity_obj.receive(new_len / 2)
                pkt = pkt_rem + pkt
            else:
                pkt = connectivity_obj.receive(pkt_size)
                # exit the code if the packet is detecting wrong continuously for more than 5 times
                count += 1
                if count > 5:
                    tkMessageBox.showinfo("Oops",
                                          "Something went wrong please restart the device and run the process again !")
                    sys.exit(1)
        except KeyboardInterrupt:
            print "Error"
            cmd = [0x32, 0x00, 0x32]
            connectivity_obj.send(cmd)
            cmd = [0x22, 0x00, 0x22]
            connectivity_obj.send(cmd)
            connectivity_obj.close()
            sys.exit(1)

    else:
        # stopping the sensor to send the inertial data
        cmd = [0x32, 0x00, 0x32]
        connectivity_obj.send(cmd)
        cmd = [0x22, 0x00, 0x22]
        connectivity_obj.send(cmd)
        break
try:
    # stopping the sensor to send the inertial data
    cmd = [0x32, 0x00, 0x32]
    connectivity_obj.send(cmd)
    cmd = [0x22, 0x00, 0x22]
    connectivity_obj.send(cmd)
except:
    pass

time.sleep(1)
if log_data:
    if outfile:
        outfile.close()
    if temp_file:
        temp_file.close()
if os.path.isfile("stop"):
    os.remove("stop")

stop = time.time()
time_taken = stop - start
isRunning = False
if d_graph == "OFF LINE" and len(pkt_number):  # to display the graph off line
    log_file = open(temp_filename)
    all_imu_data = []
    for i_imu in range(0, num_of_selected_imu * NUM_AXIS):
        all_imu_data.append([])
    while pkt != '':
        pkt = log_file.readline()
        cols = pkt.split()
        if len(cols) == 0:
            break

        # counter = 2
        for col_cnter in range(1, len(cols)):
            all_imu_data[col_cnter - 1].append(
                float(cols[col_cnter].strip()))  # store all data to list of lists(9 axis)

    log_file.close()

    app = QtGui.QApplication([])
    win = pg.GraphicsWindow(title="Calibrated Data")
    win.resize(1000, 600)
    win.setWindowTitle('RAW Data: Plotting')

    list_pt_graph = []
    num_plot = 0
    for options in plot_option:  # create subplot for acclerometer , gyroscope and magnometer if selected
        if num_plot > 0:
            win.nextRow()
        num_plot += 1

        pt_desc = plot_descs[options]
        pt_graph = win.addPlot(title=pt_desc[0])
        pt_graph.setLabel('left', pt_desc[1], units=pt_desc[2])
        pt_graph.showGrid(x=True, y=True)
        list_pt_graph.append(pt_graph)

    list_pt_graph[-1].setLabel('bottom', 'TimeSpan : %d' % time_taken, units='seconds')

    s_cntry = 0
    color = ['b', 'r', 'g']  # blue for x axis, red color for y axis and green color for z axis
    for s_imu in range(0, len(binary_string)):  # plot each axis from each imu if selected
        if binary_string[s_imu] == '1':
            acc_gyro_mag = select_acm[s_imu]
            for s_a_g in range(0, NUM_AXIS):
                if acc_gyro_mag[s_a_g] == '0':
                    continue
                which_colour = s_a_g % 3
                which_plt = int(s_a_g / 3)
                try:
                    list_pt_graph[which_plt].plot(all_imu_data[s_cntry * NUM_AXIS + s_a_g], pen=color[which_colour])
                except IndexError as e:
                    list_pt_graph[which_plt - 1].plot(all_imu_data[s_cntry * NUM_AXIS + s_a_g], pen=color[which_colour])
            s_cntry += 1

if len(pkt_number) > 0:  # to show dialog on completion
    root = Tk()
    root.withdraw()
    if d_graph:
        tkMessageBox.showinfo("Done", "Total %d packets were received in %.2f seconds" % (pkts, time_taken))
    else:
        tkMessageBox.showinfo("Done", "Data has been written in %s .\n Total %d packets were received in %.2f seconds" %
                              (dlog_file, pkts, time_taken))

if temp_filename and os.path.isfile(temp_filename):
    os.remove(temp_filename)

if app:
    sys.exit(app.exec_())
