
import math
import sys
import struct
import time
import pexpect

def create_ack(pkt_num1, pkt_num2):
	ack = []
	ack.append(1)
	ack.append(pkt_num1)
	ack.append(pkt_num2)
	ack.append((1 + pkt_num1 + pkt_num2) / 256)
	ack.append((1 + pkt_num1 + pkt_num2) % 256)
	return ack


def parse_pkt(buffer):
    pkt_info = ()
    payload = []
    start_code =  int(buffer[0:2],16)
    pkt_num1 = int(buffer[2:4],16)
    pkt_num2 = int(buffer[4:6],16)
    payload_length = int(buffer[6:8],16)
    if (start_code != 0xAA):
        print "Error: Failed to detect header at packet start"
        valid = False

    else:
        valid = True
        step_count = int(buffer[121:125],16)
        checksum = int(buffer[125:128],16)
        pkt_info = (pkt_num1, pkt_num2, step_count, checksum)
        payload = buffer[8:120]
    return valid, pkt_info, payload


def calc_disp(sensor_data, theta):
        d0 = struct.unpack("!f",sensor_data[0:8].decode("hex"))[0]
        d1 = struct.unpack("!f",sensor_data[8:16].decode("hex"))[0]
        d2 = struct.unpack("!f",sensor_data[16:24].decode("hex"))[0]
        d3 = struct.unpack("!f",sensor_data[24:32].decode("hex"))[0]

        sin_theta = math.sin(theta)
        cos_theta = math.cos(theta)

        dx = d0 * cos_theta + d1 * sin_theta
        dy = d0 * sin_theta + d1 * cos_theta
        dz = d2
        dp = d3

        disp = math.sqrt(dx * dx + dy * dy + dz * dz)
        return dx,dy,dz,dp,disp


def calc_dist(x,y,z):
	r = x*x + y*y + z*z
	return math.sqrt(r)



# if len(sys.argv) != 3:
#     print 'Error must specify address as parameter!'
#     print 'Example: sudo python dataparsing.py 5C:31:3E:F2:16:13 test.txt'
#     sys.exit(1)
#ID = sys.argv[1]
ID = '00:A0:50:E4:74:C6'
#numbytes = sys.argv[2]
numbytes = 64*30
outfile = "text2.text"
# outfile = sys.argv[2]

# Run gatttool interactively.
gatt = pexpect.spawn('gatttool -I')

# Connect to the device.
gatt.sendline('connect {0}'.format(ID))
gatt.expect('Connection successful')
gatt.expect("\r\n", timeout=10)

gatt.sendline('char-write-cmd 0x0011 340034')
gatt.expect('')

gatt.sendline('char-read-hnd 0x000e')
gatt.expect('Characteristic value/descriptor:')
gatt.expect("\r\n", timeout=5)

packet_info = [0,0,0,0]
count = 0
pkt_len = 64
num_pkts = 0
curr_pkt = 0
prev_pkt = -1

data = []

xpos = 0.0   # x-coord in user's reference frame
ypos = 0.0   # y-coord in user's reference frame
zpos = 0.0   # z-coord in user's reference frame
phi  = 0.0   # Angular position around Z-axis in user's reference frame


# Enter main loop.
while (count < numbytes):
    gatt.sendline('char-read-hnd 0x000e')
    gatt.expect('Characteristic value/descriptor:')
    gatt.expect("\r\n", timeout=10)

    str1 = "".join(gatt.before.split())
    valid, packet_info, payload = parse_pkt(str1)
    print payload
    if (valid == False): continue

    curr_pkt = packet_info[0] * 256 + packet_info[1]
    # print "Read packet %d...Sending Ack" % curr_pkt
    ack = create_ack(packet_info[0], packet_info[1])

    gatt.sendline('char-write-cmd 0x0011 {0:02x}{1:02x}{2:02x}{3:02x}{4:02x}'.format(*ack))

    if (curr_pkt == prev_pkt): continue

    dx, dy, dz, dp, disp = calc_disp(payload, phi)
    #print payload
    #print dx,dy,dz,dp
    xpos += dx
    ypos += dy
    zpos += dz
    phi += dp

    radial_dist = calc_dist(xpos, ypos, 0)  # for now do not factor in zpos

    num_pkts += 1
    count += pkt_len
    print num_pkts
    prev_pkt = curr_pkt

    data.append(xpos)
    data.append(ypos)
    data.append(zpos)
    data.append(phi)
    data.append(disp)

#command to stop processing
gatt.sendline('char-write-cmd 0x0011 320032')

#command to stop all outputs
gatt.sendline('char-write-cmd 0x0011 220022')

out = open(outfile, "wb")
for i in range(0, num_pkts*5, 5):
	out.write(', '.join(str(x) for x in data[i:i+5]))
	out.write("\n")
out.flush()
gatt.sendline('disconnect')
print "Written into file"
out.close()
